var ball = {
    x : 0,
    y : 0,
    dx : 8,
    dy : 0,
    radius : 18,
    jumpSpeed : -16,
    xSpeed : 8,
    direction : -1,
    topBumped : false,
    botBumped : false,
    rightBumped : false,
    leftBumped : false,
    item : false,
    itemUsed : false,
    locate : function(x, y){
        this.x = x*60+30;
        this.y = y*60+30;
    },
    draw : function(){
        ctx.beginPath();
        ctx.arc(this.x, this.y, this.radius, 0, Math.PI*2);
        if(ball.item) ctx.fillStyle = "black";
        else ctx.fillStyle = "#f5f500";
        ctx.fill();
        ctx.closePath();
        if(!pause){
          if(pressed.right && !this.leftBumped && !this.itemUsed){
            this.dx = this.xSpeed;
            this.direction = 1;
          } else if(pressed.left && !this.rightBumped && !this.itemUsed){
            this.dx = -this.xSpeed;
            this.direction = -1;
          } else {
            this.dx = 0;
          }
          if(this.itemUsed){
            this.dx = this.direction * this.xSpeed * 1.5;
            if(this.topBumped) {
              this.itemUsed = false;
            }
          }
          if(this.topBumped){
              this.dy = this.jumpSpeed;
              this.topBumped = false;
          }
          if(this.botBumped){
            this.dy = Math.abs(this.dy);
            this.botBumped = false;
          }
          this.dy += 1;
          if (this.y + this.radius > canvas.height){
            restart();
          }
          ball.rightBumped = false;
          ball.leftBumped = false;
          this.x += this.dx;
          this.y += this.dy;
        }
    },
}

class Block {
  constructor(){
    this.location = [];
    this.broken = [];
    this.motion = [];
  }
  locate(x,y,mode='block'){
    //32*60 width 27*60 height
    //mode : block breakable obstacle item
    x *= 60;
    y *= 60;
    var width = 60;
    var height = 60;
    this.location[this.location.length] = [x,y,width,height,mode];
  }
  draw(){
    for(var i = 0; i < this.location.length; i++){
      if(!this.broken[i]){
        /************************************************************
         * drawing block
        */
        ctx.beginPath();
        ctx.rect(this.location[i][0],this.location[i][1],this.location[i][2],this.location[i][3]);
        if(this.location[i][4] == 'breakable') ctx.fillStyle = "#b4c8ff";
        else if(this.location[i][4] == 'item') ctx.fillStyle = "#dcdc00";
        else ctx.fillStyle = "#f5f5f5";
        ctx.fill();
        ctx.closePath();
        ctx.beginPath();
        ctx.lineWidth = 5;
        if(this.location[i][4] == 'item') ctx.strokeStyle = "rgba(0,0,0,0)"
        else ctx.strokeStyle = "black";
        if(this.location[i][4] == 'breakable' || this.location[i][4] == 'obstacle'){
          ctx.setLineDash( [8,8] );
        } else {
          ctx.setLineDash( [0,0] )
        }
        ctx.strokeRect(this.location[i][0]+3,this.location[i][1]+3,this.location[i][2]-3,this.location[i][3]-3);
        ctx.closePath();
        if(this.location[i][4] == 'obstacle'){
          ctx.beginPath();
          ctx.setLineDash( [0,0])
          ctx.lineWidth = 3;
          ctx.moveTo(this.location[i][0]+36,this.location[i][1]+6);
          ctx.lineTo(this.location[i][0]+30,this.location[i][1]+24);
          ctx.lineTo(this.location[i][0]+48,this.location[i][1]+30);
          ctx.lineTo(this.location[i][0]+24,this.location[i][1]+54);
          ctx.lineTo(this.location[i][0]+30,this.location[i][1]+36);
          ctx.lineTo(this.location[i][0]+12,this.location[i][1]+30);
          ctx.lineTo(this.location[i][0]+36,this.location[i][1]+6);
          ctx.stroke();
          ctx.fillStyle = 'yellow';
          ctx.fill();
          ctx.closePath();
        } 
        if(this.location[i][4] == 'item'){
          ctx.beginPath();
          ctx.arc(this.location[i][0]+44,this.location[i][1]+20, 11, 0, Math.PI*2);
          ctx.fillStyle = 'rgba(0,0,0)';
          ctx.fill();
          ctx.closePath();

          ctx.beginPath();
          ctx.arc(this.location[i][0]+33,this.location[i][1]+30, 8, 0, Math.PI*2);
          ctx.fillStyle = 'rgba(0,0,0,0.8)';
          ctx.fill();
          ctx.closePath();

          ctx.beginPath();
          ctx.arc(this.location[i][0]+26,this.location[i][1]+40, 5, 0, Math.PI*2);
          ctx.fillStyle = 'rgba(0,0,0,0.6)';
          ctx.fill();
          ctx.closePath();

          ctx.beginPath();
          ctx.arc(this.location[i][0]+23,this.location[i][1]+50, 2, 0, Math.PI*2);
          ctx.fillStyle = 'rgba(0,0,0,0.4)';
          ctx.fill();
          ctx.closePath();
        }

        /********************************************************
         * calculating bump
         */
        if(ball.x + ball.radius/3 > this.location[i][0] && ball.x - ball.radius/3 < this.location[i][0] + this.location[i][2]){
          if(ball.y + ball.dy > this.location[i][1] && ball.y + ball.radius < this.location[i][1] + this.location[i][3]){
            if(this.location[i][4]=='breakable') {
              ball.topBumped = true;
              this.broken[i] = true;
            }
            else if(this.location[i][4] == 'obstacle') restart();
            else if(this.location[i][4] == 'item') {
              this.broken[i] = true;
              ball.item = true;
            }
            else ball.topBumped = true;
          } else if(ball.y + ball.dy < this.location[i][1] + this.location[i][3] &&
              ball.y - ball.radius > this.location[i][1]){
            if(this.location[i][4] == 'obstacle') restart();
            else if(this.location[i][4] == 'item') {
              this.broken[i] = true;
              ball.item = true;
            }
            else ball.botBumped = true;
          }
        }else if(ball.y > this.location[i][1] && ball.y < this.location[i][1] + this.location[i][3]){
          if(ball.x + ball.radius + ball.dx > this.location[i][0] && ball.x - ball.radius - ball.dx < this.location[i][0]+this.location[i][2]){
            if(ball.x < this.location[i][0]){
              if(this.location[i][4] == 'block' || this.location[i][4] == 'breakable') ball.leftBumped = true;
              else if(this.location[i][4] == 'obstacle') restart();
              else if(this.location[i][4] == 'item') {
                this.broken[i] = true;
                ball.item = true;
              }
            } else if(ball.x > this.location[i][0] + this.location[i][2]){
              if(this.location[i][4] == 'block' || this.location[i][4] == 'breakable') ball.rightBumped = true;
              else if(this.location[i][4] == 'obstacle') restart();
              else if(this.location[i][4] == 'item'){
                this.broken[i] = true;
                ball.item = true;
              }
            }
          } else {
            ball.rightBumped = false;
            ball.leftBumped = false;
          }
        }
        /***************************************
         * moving block control
         */
        if(this.motion[i] !== undefined){
          if(this.motion[i].direction == 'x'){
            if(Math.max(this.motion[i].target1,this.motion[i].target2) < this.location[i][0]){  
              this.motion[i].way = -1;
            } else if(Math.min(this.motion[i].target1,this.motion[i].target2) > this.location[i][0]){
              this.motion[i].way = 1;
            }
            this.location[i][0] += this.motion[i].speed * this.motion[i].way;
          }else if(this.motion[i].direction == 'y'){
            if(Math.max(this.motion[i].target1,this.motion[i].target2) < this.location[i][1]){  
              this.motion[i].way = -1;
            } else if(Math.min(this.motion[i].target1,this.motion[i].target2) > this.location[i][1]){
              this.motion[i].way = 1;
            }
            this.location[i][1] += this.motion[i].speed * this.motion[i].way;
          }
        }
      }
    }
  }

  moving(x,y,speed,range,direction,mode){
    var target1, target2;
    range *= 60;
    if(direction != 'x' && direction != 'y'){
      throw "direction should be x or y";
    } else {
      this.locate(x,y,mode);
      if(direction=='x'){
        target1 = this.location[this.location.length-1][0];
        target2 = this.location[this.location.length-1][0] + range;
      } else if(direction=='y'){
        target1 = this.location[this.location.length-1][1];
        target2 = this.location[this.location.length-1][1] + range;
      }
      this.motion[this.location.length-1] = {speed : speed, target1 : target1,
         target2 : target2, direction : direction, way : -1};
    }
  }
}

class Star{
  constructor(){
    this.location = [];
    this.taken = [];
  }
  locate(x,y){
    this.location[this.location.length] = [x*60+30,y*60+30];
    this.taken[this.taken.length] = false;
  }
  draw(){
    for(var i = 0; i < this.location.length; i++){
      if(!this.taken[i]){
        var cx = this.location[i][0];
        var cy = this.location[i][1];
        var outerRadius = 20;
        drawStar(this.location[i][0],this.location[i][1],outerRadius, 8);
        var distance = Math.sqrt((ball.x - cx)*(ball.x - cx) + (ball.y - cy)*(ball.y - cy));
        if(distance < outerRadius+5){
          this.taken[i] = true;
        }
      }
      var completed = this.taken.every(value => value == true);
      if(completed && (this.taken.length != 0)){
        levelCompleted[status] = mode;
        end = true;
      }
    }
  }
}

function drawStar(cx,cy,outerRadius,innerRadius){
    var spikes = 5;
    var rot = Math.PI / 2 * 3;
    var x = cx;
    var y = cy;
    var step = Math.PI / spikes;

    ctx.strokeSyle = "#000";
    ctx.beginPath();
    ctx.moveTo(cx, cy - outerRadius)
    for (j = 0; j < spikes; j++) {
        x = cx + Math.cos(rot) * outerRadius;
        y = cy + Math.sin(rot) * outerRadius;
        ctx.lineTo(x, y)
        rot += step

        x = cx + Math.cos(rot) * innerRadius;
        y = cy + Math.sin(rot) * innerRadius;
        ctx.lineTo(x, y)
        rot += step
    }
    ctx.lineTo(cx, cy - outerRadius)
    ctx.closePath();
    ctx.lineWidth = 5;
    ctx.strokeStyle = "#d3d300";
    ctx.stroke();
    ctx.fillStyle='yellow';
    ctx.fill();
}